import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format
} from 'd3';

// Colors for the sections in Pie Chart
const sectionColors = [
  		'silver', 'lightblue', 'grey', 'lightgreen', 'yellow',
      'gold', 'pink', 'orange', 'seagreen', 'purple'
		  ];

// Data set URL
const csvUrl = 'https://gist.githubusercontent.com/Gogineni44/0f135d919751b2db1834883318f3d5fa/raw/8321e99e7c5f652594cf3342e5c1073b689b1826/songs.csv';

// Function to fetch data from the URL
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

// Select the SVG from the HTML body
const svg = select("svg");
const width = + svg.attr('width');
const height = + svg.attr('height');

// Format axes text
const xAxisTickFormat = number =>
  	format('.4~g')(number);

// Render Pie Chart
const render = (data, total) => {
  const pieData = pie().value(d=>d.duration)(data);
  const colors = scaleOrdinal()
  	.domain(data)
  	.range(sectionColors);
  
  // Object for generating Arcs of the sections
  const arcGenerator = d3.arc()
  .innerRadius(0)
  .outerRadius(250);

  // Segments are the partition in the Pie Chart
	const segments = arc()
  	.innerRadius(0)
  	.outerRadius(175)
  	.padAngle(0)
  	.padRadius(0);
  
  const sections = svg.append("g")
  	.attr("transform", `translate(250,250)`)
  	.selectAll("path").data(pieData);
  
  sections.enter()
    .append("path")
    .attr("d", segments)
  	.attr("fill", d => colors(d.data.duration));
  
  // Render text on each section
  sections
    .enter()
    .append('text')
    .text(d => xAxisTickFormat((d.data.duration)))
    .attr("transform", function(d) { return "translate(" + arcGenerator.centroid(d) + ")";  })
    .style("text-anchor", "middle")
  	.style('fill', 'black')
    .style("font-size", 15);
  
  // Render legend for the data
  const legends = svg.append("g")
  	.attr("transform", "translate(500,100)")
    .selectAll(".legends").data(pieData);
  
  const legend = legends.enter().append("g").classed(".legends",true)
  	.attr("transform", (d,i)=>{
      return `translate(0,${(i+1)*30})`;
    });
  
  // Formats the legend for duration
  legend.append("rect").attr("width",20).attr("height",20)
  	.attr("fill", d => colors(d.data.duration));
  
  legend.append("text")
  	.attr("x", 250)
  	.attr("y", 15)
  	.attr("class","legend_value")
  	.text(d=>xAxisTickFormat((d.data.duration)));
  
  // Formats the legend for song
  legend.append("text")
  	.attr("x", 25)
  	.attr("y", 15)
  	.attr("class","legend_text")
  	.text(d => d.data.song);
  
}

// Main function
const run = async () => {
  let data = await fetchData()
  let total = 0
  data.forEach(d=>{
    d.duration = +d.duration;
    total += d.duration
  });
	render(data,total);
};
run();